from clyde import Command, Option, ManpageFormatter, ParseError, command, mixin


class Program(Command):
    """Main program.

    Lorem ipsum dolor sit amet, consectetur adipiscing elit,
    sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
    Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris.
    """

    # Public

    meta_formatter = ManpageFormatter

    # Mixins

    @mixin(require=ParseError)
    def process_parse_error(self, exception):
        print('Processing parse error..')

    @mixin
    def initiate_logging(self):
        print('Initiating logging..')

    @mixin(require='help')
    def print_help(self):
        print('Printing help..')
        print(self.meta_format('help'))
        exit()

    @mixin(require=Exception)
    def process_exception(self, exception):
        print('Processing exception..')

    # Options

    help = Option(
        action='store_true',
        flags=['-h', '--help'],
        help='Print help and exit.',
    )

    verbose = Option(
        action='store_true',
        flags=['-v', '--verbose'],
        help='Enable verbose mode.',
    )

    # Subcommands

    class build(Command):
        """Build something.

        Lorem ipsum dolor sit amet, consectetur adipiscing elit,
        sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
        Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris.
        """

        # Public

        def meta_execute(self, source, target='target', *pathes):
            print('Building..', source, target, pathes, sep=', ')

        # Options

        date = Option(
            action='store_true',
            flags=['-d', '--date'],
            help='Today date.')

        time = Option(
            action='store_true',
            flags=['-t', '--time'],
            help='Today time.')

    @command
    def deploy(self, count: int):
        """Deploy something.
        """
        print('Deploying..', type(count))


if __name__ == '__main__':
    program = Program()
    program()
